<?php

namespace App\Http\Controllers;

use DateTime;
use App\Betting;
use App\GameResult;
use App\Transaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Cron\CronExpression;
use Carbon\Carbon;

class TradeController extends Controller
{
    protected $bet_close = [
        1 => 5,
        3 => 5,
        5 => 5,
    ];

    protected $max_amount = [
        1 => 165,
        3 => 412.5,
        5 => 495,
    ];

    protected $config = [
        'BTCUSDT' => [
            'chart_type' => 'widget',
            'chart_symbol' => 'BINANCE:BTCUSDT',
            'description' => 'Binance BTC/USDT',
            'decimal_length' => '2',
        ],
        'GBPAUD' => [
            'chart_type' => 'self',
            'chart_symbol' => 'GBPAUD',
            'description' => 'Live-Rates GBP/AUD',
            'decimal_length' => '5',
        ],
    ];

    public function symbolProcess($symbol, $abortIfNotExists = false)
    {
        if (!in_array($symbol, ['BTCUSDT', 'GBPAUD'])) {
            if ($abortIfNotExists) {
                abort(404);
            }

            return 'BTCUSDT';
        }

        return $symbol;
    }


    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(Request $request, $symbol, $minutes = 1)
    {
        $symbol = $this->symbolProcess($symbol);

        $bettings = $request->user()->bettings()->whereSymbol($symbol)->whereType($minutes)->latest()->get();

        return view('trade', [
            'config' => $this->config[$symbol],
            'symbol' => $symbol,
            'minutes' => $minutes,
            'bettings' => $bettings,
        ]);
    }

    public function listTrades(Request $request, $minutes = NULL)
    {
        $date = $request->date ?: date('Y-m-d');

        if ($minutes) {
            if ($request->has('all')) {
                $bettings = $request->user()->bettings()->whereType($minutes)->latest()->get();
            } else {
                $bettings = $request->user()->bettings()->whereDate('created_at', $date)->whereType($minutes)->latest()->get();
            }
        } else {
            if ($request->has('all')) {
                $bettings = $request->user()->bettings()->latest()->get();
            } else {
                $bettings = $request->user()->bettings()->whereDate('created_at', $date)->latest()->get();
            }
        }

        return view('trades', [
            'config' => $this->config,
            'bettings' => $bettings,
        ]);
    }

    public function info(Request $request, $symbol, $minutes)
    {
        $symbol = $this->symbolProcess($symbol, true);

        $total_round = 86400 / ($minutes * 60); // 1: 1440 / 3: 480 / 5: 288
        $pad_sec = $minutes * 60;
        $bet_round = ceil(Carbon::now()->addSeconds($pad_sec)->diffInSeconds((new Carbon('+'.$pad_sec.' seconds'))->startOfDay()) / $pad_sec) % $total_round;
        $play_round = ceil(Carbon::now()->diffInSeconds((new Carbon('-'.$pad_sec.' seconds'))->startOfDay()) / $pad_sec) % $total_round;

        if ($bet_round == 0) {
            $bet_round = date('ymd', time() + $pad_sec).str_pad($total_round, 4, '0', STR_PAD_LEFT);
        } else {
            $bet_round = date('ymd', time() + $pad_sec).str_pad($bet_round, 4, '0', STR_PAD_LEFT);
        }

        if ($play_round == 0) {
            $play_round = date('ymd', time() - $pad_sec).str_pad($total_round, 4, '0', STR_PAD_LEFT);
        } else {
            $play_round = date('ymd', time() - $pad_sec).str_pad($play_round, 4, '0', STR_PAD_LEFT);
        }

        $startTime = CronExpression::factory('*/'.$minutes.' * * * *')->getNextRunDate()->format('Y-m-d H:i:s');
        $playingTime = CronExpression::factory('*/'.$minutes.' * * * *')->getPreviousRunDate('now', 0, true)->format('Y-m-d H:i:s');

        $bet_round_time_range = [
            (new Carbon($startTime))->format('H시 i분'),
            (new Carbon($startTime))->addMinutes($minutes)->format('H시 i분')
        ];

        $play_round_time_range = [
            (new Carbon($playingTime))->format('H시 i분'),
            (new Carbon($playingTime))->addMinutes($minutes)->format('H시 i분')
        ];

        $remain_dateinterval = CronExpression::factory('*/'.$minutes.' * * * *')->getNextRunDate()->diff(new DateTime);
        $remain_seconds = ($remain_dateinterval->i * 60) + $remain_dateinterval->s;

        $maintenance_mode = file_exists(storage_path('framework/bet_close_'.strtolower($symbol)));

        $response = [
            'symbol' => $symbol,
            'time' => date('Y-m-d H:i:s'),
            'playing_round' => $play_round,
            'playing_round_time_range' => $play_round_time_range,
            'bet_round' => $bet_round,
            'bet_round_time_range' => $bet_round_time_range,
            'bet_close' => $remain_seconds - $this->bet_close[$minutes],
            'maintenance_mode' => $maintenance_mode,
        ];

        return response($response);
    }

    public function listGameResultJson(Request $request, $symbol, $minutes)
    {
        $symbol = $this->symbolProcess($symbol, true);

        $amount = $request->amount ? ($request->amount > 1440 ? 1440 : $request->amount) : 1;
        $list = GameResult::whereSymbol($symbol)->whereType($minutes)->orderBy('round', 'desc')->take($amount)->get();

        return response($list);
    }

    public function betGame(Request $request, $symbol, $minutes)
    {
        // validate balance, amount, pick, time
        $symbol = $this->symbolProcess($symbol, true);

        $maintenance_mode = file_exists(storage_path('framework/bet_close_'.strtolower($symbol)));
        if ($maintenance_mode) {
            return response([
                'message' => '현재 거래 서비스 점검중입니다'
            ], 503);
        }

        if ($minutes != 1 && $minutes != 3 && $minutes != 5) {
            return response([
                'message' => '시간이 올바르지 않습니다'
            ], 403);
        }

        $remain_dateinterval = CronExpression::factory('*/'.$minutes.' * * * *')->getNextRunDate()->diff(new DateTime);
        $remain_seconds = ($remain_dateinterval->i * 60) + $remain_dateinterval->s;

        if ($remain_seconds <= $this->bet_close[$minutes]) {
            return response([
                'message' => '주문이 마감되었습니다. 다음 회차에 참여해주세요'
            ], 403);
        }

        if ((($minutes * 60) - $this->bet_close[$minutes]) <= $remain_seconds) {
            return response([
                'message' => '다음 회차 준비중입니다. 잠시 후 다시 시도해주세요'
            ], 403);
        }

        $total_round = 86400 / ($minutes * 60); // 1: 1440 / 3: 480 / 5: 288
        $pad_sec = $minutes * 60;

        $round = ceil(Carbon::now()->addSeconds($pad_sec)->diffInSeconds((new Carbon('+'.$pad_sec.' seconds'))->startOfDay()) / $pad_sec) % $total_round;

        if ($round == 0) {
            $round = date('ymd', time() + $pad_sec).str_pad($total_round, 4, '0', STR_PAD_LEFT);
        } else {
            $round = date('ymd', time() + $pad_sec).str_pad($round, 4, '0', STR_PAD_LEFT);
        }

        $user = $request->user();

        if (Betting::whereUserId($user->id)->whereType($minutes)->whereRound($round)->exists()) {
            return response([
                'message' => '이미 이번 회차에 참여하셨습니다'
            ], 403);
        }

        $amount = $request->amount;

        if ($amount <= 0 || $amount > $this->max_amount[$minutes]) {
            return response([
                'message' => '수량이 올바르지 않습니다'
            ], 403);
        }

        $amount_to_krw = bcmul($amount, 10000); // 1.00x

        if ($user->balance < $amount_to_krw) {
            return response([
                'message' => '보유 금액이 부족합니다'
            ], 403);
        }

        if (!in_array($request->pick, ['up', 'down'])) {
            return response([
                'message' => '선택이 올바르지 않습니다'
            ], 403);
        }

        $betting = new Betting;
        $betting->user_id = $user->id;
        $betting->symbol = $symbol;
        $betting->type = $minutes;
        $betting->round = $round;
        $betting->pick = $request->pick;
        $betting->amount = $amount;

        $betting->save();

        $tx = new Transaction;

        $tx->user_id = $user->id;
        $tx->type = 'withdraw_game_betting';
        $tx->amount = $amount_to_krw;
        $tx->info = 'Betting id: '.$betting->id;
        $tx->status = 'success';

        $tx->save();

        $user->decrement('balance', $tx->amount);

        return response([
            'message' => '주문이 완료되었습니다'
        ], 201);
    }
}
